


# EitilWrapper

EitilWrapper wraps core rails operations with extended utilities – such as routing, jobs and decoraters.






## EitilWrapper::Callbacks

```ruby

require "eitil_wrapper/callbacks"

```

Callback helper methods are created for use within a model's callbacks. You can use them as callback conditionals, e.g. "after_save :do_this, if :doing_became_true". Which scopes are generated depends on the datatype of a column. The methods currently generated are:

```ruby
# require "eitil_wrapper/callbacks/helper_methods"

# columns of datatype: boolean
.{column_name}_became_true
.{column_name}_becomes_true
.{column_name}_to_true # works for both become and became

.{column_name}_became_false
.{column_name}_becomes_false
.{column_name}_to_false # works for both become and became
```





## EitilWrapper::Decorators

```ruby

require "eitil_wrapper/decorators"

```

The Eitil decorator wrappers help you to standardize the calling of the right decorator method from within your controller action. Basically it provides you with a decorate macro in each controller.

```ruby
decorate(dec_item, dec_method: nil, dec_class: nil, **dec_kwargs)
```

- dec_item is the instance that will be decorated
- dec_method enabled you to set the desired decorator method. If not provided, it will look into the request params: if params["isMobile"] is present it will call .app, if params["isWeb"] is present it will call :app. If neither is provided in the params, it will call the default method :base.
- dec_class enables you to overwrite the decorator class that will be called. If not provided, the decorator class will be inferred from the instance model's classname (User => UserDecorator).
- dec_kwargs enables you to provide additional arguments, which will be passed to your decorator method.


### Configuration

1. Your decorator classes should inherit from EitilWrapper::ApplicationDecorator.
2. Your controllers should inherit the module EitilWrapper::ControllerDecorator, through inclusion in a superclass.
3. If you set controller ivars for each request, you can make them available in your decorators by providing Eitil with the names of your ivars as an array of symbols:

```ruby
# initializers/eitil.rb

Eitil.set_config do |config|
  config.controller_ivars = [:user, :env]
end
```





## EitilWrapper::Jobs

```ruby

require "eitil_wrapper/jobs"

```

The Eitil jobs wrapper enables you to create perform_now and perform_later jobs on the fly, without the need to create a new class and file. 

The macro new_job accepts a :method as argument and defines a new method :method_job. The newly defined :method_job, when called, performs the orginal :method in the background. The new_job macro accepts both instance methods and singleton methods, which are defined within there own method scope. 

In contrast, the new_job_now macro defines a new :method_job_now method, which performs in the foreground.

```ruby
# require "eitil_wrapper/jobs/new_job"

new_job(_method, *args, **kwargs)
# assuming a method :hello_world, call as: new_job :hello_world
# => defines :hello_world_job
```

```ruby
# require "eitil_wrapper/jobs/new_job_now"

new_job_now(_method, *args, **kwargs)
# assuming a method :hello_world, call as: new_job_now :hello_world
# => defines :hello_world_job_now
```





## EitilWrapper::Routes

```ruby

require "eitil_wrapper/routes"

```

The Eitil router wrapper enables you to easily create new routes that can be shared among controllers. In that sense, it is as a custom extension of the built-in resourceful routes.

### New route

The first macro, .new_route, enables you to create new route objects which may be called and used by different controllers.

```ruby
# require "eitil_wrapper/routes/extended_resources"

new_route(verb, _method, action, scope)
# call as: new_route :post, :attachment, "add_attachment", :member
```

- verb refers to the http_verb, e.g. :put.
- method refers to the given http method (e.g. 'users/:id'). Additionally, this is the name you will refer to in order to include the route into a controller. Therefore the argument should be unique among all other routes.
- action refers to the controller action to which you want to redirect to request, e.g. :update_user.
- scope refers to the url and can be set to either :member or :collection.

### Add route

The second macro, extended_resources, enables you to create all standard resources for a controller and add your own standardized routes. This macro works the same as the built-in :resources, except that you can pass more arguments to only: [], which adds the routes to your controller.

```ruby
# require "eitil_wrapper/routes/extended_resources"

extended_resources(controller, **kwargs)
# call as: extended_resources :posts, only: [:create, :update, :attachment]
```





## EitilWrapper::Scopes

```ruby

require "eitil_wrapper/scopes"

```

Scopes are generated through the columns of your model's database table. Which scopes are generated depends on the datatype of a column. For example, if your User model with table users has a column is_manager, a scope :is_manager_true is generated for your User model. A scope method is defined only if it does not already responds to another method of the same name. The scopes generated are:

```ruby
# require "eitil_wrapper/scopes/default_scopes"

# columns of datatype: boolean
.{column_name}_true
.{column_name}_false

# columns of datatype: datetime
.{column_name}_today      
.{column_name}_past       
.{column_name}_future             
.{column_name}_on_date(date)    
.{column_name}_before_date(date)
.{column_name}_after_date(date) 
.{column_name}_between_dates(start_date, end_date)
.{column_name}_oldest_first
.{column_name}_newest_first

# columns of datatype: date
.{column_name}_today      
.{column_name}_past       
.{column_name}_future             
.{column_name}_on_date(date)    
.{column_name}_before_date(date)
.{column_name}_after_date(date) 
.{column_name}_between_dates(start_date, end_date)
.{column_name}_oldest_first
.{column_name}_newest_first

# columns of datatype: integer
.{column_name}_equal_to(number)      
.{column_name}_lower_than(number)
.{column_name}_higher_than(number)
.{column_name}_between(min, max)
.{column_name}_ascending
.{column_name}_descending

# columns of datatype: float
.{column_name}_equal_to(number)      
.{column_name}_lower_than(number)
.{column_name}_higher_than(number)
.{column_name}_between(min, max)
.{column_name}_ascending
.{column_name}_descending
```





## EitilWrapper::RequestLogger

```ruby

require "eitil_wrapper/request_logger"

```

The RequestLogger wrapper logs request params in /log/request_logger.log, which offers the opportunity to pry into webhooks while developing. In order to track a controller, simply add the following. This calls a background job which writes to the file.

```ruby

include EitilWrapper::RequestLogger::ControllerMixin
before_action :log_request

```





